from typing import Sequence

def compute_statistics_fast(values: Sequence[float]) -> dict[str, float | int]: ...
def compute_interarrival_times_fast(timestamps: Sequence[float]) -> list[float]: ...
def compute_percentiles_fast(
    values: Sequence[float],
    percentiles: Sequence[int] = ...,
) -> dict[str, float]: ...
